/* ----------------------------------------------------------------------------
**      keytree.c  Implementation of keyboard lookup tables, see doc/en
** ----------------------------------------------------------------------------
**      TGT                                       TextGUIToolkit Library
**      Copyright (C)   2000-2001       Mateusz Golicz and Tomasz Sterna
**
**      LGPL license
**
** ----------------------------------------------------------------------------
**      MAINTAINER      Mateusz Golicz
**
**      Log:
**      - cleanup
**
*/

#include "tgt_keyboard.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


/*
 * tgt_allockeylookuptable: Allocate memory for a lookup table, and set all
 *                          keys to KEYN_NONE type.
 */
struct tgt_keynode * tgt_allockeylookuptable(void)
{
    struct tgt_keynode * ret;
    int i;
    ret=(struct tgt_keynode*) malloc(sizeof(struct tgt_keynode)*256);
    for(i=0;i<256;i++) ret[i].type=TGT_KEYN_NONE;
    return(ret);
}

/*
 * tgt_initroottable: Allocate root lookup table, in which type(i) equals
 *                    KEYN_KEY and value(i) = i
 *                    
 */
struct tgt_keynode * tgt_initroottable(void)
{
    int i;
    struct tgt_keynode *mt;
    mt=tgt_allockeylookuptable();
    for(i=0;i<256;i++)
    {
	mt[i].type=TGT_KEYN_KEY;
	mt[i].value=(void*) i;
    }
    return(mt);
}

/*
 * tgt_destroylookuptable: Free memory associated with a lookup table
 *                    
 */
void tgt_destroylookuptable(struct tgt_keynode * lt)
{
    int i;
    
    for(i=0;i<256;i++)
	if(lt[i].type==TGT_KEYN_LOOKUPTABLE) 
	    tgt_destroylookuptable(lt[i].value);
	    /* recursively destroy all subtables */

    free(lt);
}

/*
 * tgt_addkeyseq: Add a multiple key sequence into a set of lookup-tables
 *                    
 */
void tgt_addkeyseq(struct tgt_keynode *l_main_table,unsigned char *seq,int c)
{
    int k,l;
    struct tgt_keynode *lt;
    struct tgt_keynode *nn;
    int ln,i;
    l = strlen(seq);
    lt = l_main_table;
    for(k=0;k<(l-1);k++) /* foreach character in our sequence, except the last one */
    {
	/* Get the next character */
	i=seq[k];
	/* Check existing keytable record for it */
	ln=lt[i].type;

	if(ln==TGT_KEYN_KEY || ln==TGT_KEYN_NONE)
	{
/* there's no record, or there's a conflict (in that case we overwrite the older sequence */
/* allocate a new table */
	    nn=tgt_allockeylookuptable();
	    lt[i].type=TGT_KEYN_LOOKUPTABLE; /* and setup record for our character */
	    lt[i].value=nn;
/* this table'll be used in the next iteration */
	    lt=nn;
	}
	else /* there already exists a lookup-table for that character */
	    lt=(struct tgt_keynode*) lt[i].value; /* so it'll be used in the next iteration */
    }

    i = seq[l-1]; /* the last character */
    /* in case of conflict, we overwrite the older sequence */
    if(lt[i].type==TGT_KEYN_LOOKUPTABLE) tgt_destroylookuptable(lt[i].value);
    lt[i].type=TGT_KEYN_KEY; lt[i].value=(void*) c;
}
