/* ----------------------------------------------------------------------------
**      print.c  functions for handling direct screen printing
** ----------------------------------------------------------------------------
**      TGT                                       TextGUIToolkit Library
**      Copyright (C)   2000-2001       Mateusz Golicz and Tomasz Sterna
**
**      LGPL license
**
** ----------------------------------------------------------------------------
**      MAINTAINER      Tomasz Sterna
**
**      Log:
**      - comments
**
*/


#include "tgt.h"
#include <stdio.h>


/*
 * tgt_mvcladdstr - move-color-add-string
 *                  move cursor to x,y
 *                  change foreground-color to fg and background-color to bg
 *                  and put string 'str' to screen
 */
void tgt_mvcladdstr(int x, int y, int fg, int bg, const char *str)
{
    tgt_chattr(TGT_TA_CURSOR,x,y);
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0,0); else tgt_chattr(TGT_TA_NORMAL,0,0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_COLORS,fg,bg);
    printf("%s",str);
    fflush(stdout);
}

/*
 * tgt_mvcladdch - move-color-add-character
 *                 move cursor to x,y
 *                 change foreground-color to fg and background-color to bg
 *                 and put character 'chr' to screen
 */
void tgt_mvcladdch(int x, int y, int fg, int bg, const char chr)
{
    tgt_chattr(TGT_TA_CURSOR,x,y);
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0,0); else tgt_chattr(TGT_TA_NORMAL,0,0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_COLORS,fg,bg);
    putchar(chr);
    fflush(stdout);
}

/*
 * tgt_claddstr - color-add-string
 *                change foreground-color to fg and background-color to bg
 *                and put string 'str' to screen
 */
void tgt_claddstr(int fg, int bg, const char *str)
{
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0,0); else tgt_chattr(TGT_TA_NORMAL,0,0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_COLORS,fg,bg);
    printf("%s",str);
    fflush(stdout);
}

/*
 * tgt_claddch - color-add-character
 *               change foreground-color to fg and background-color to bg
 *               and put character 'chr' to screen
 */
void tgt_claddch(int fg, int bg, const char chr)
{
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0, 0); else tgt_chattr(TGT_TA_NORMAL,0, 0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_COLORS,fg, bg);
    putchar(chr);
    fflush(stdout);
}

/*
 * tgt_chclr - change-color
 *             change current printing color,
 *             foreground-color to fg and background-color to bg
 */
void tgt_chclr(int fg, int bg)
{
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0, 0); else tgt_chattr(TGT_TA_NORMAL,0, 0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_COLORS,fg, bg);
}

/*
 * tgt_chfg - change-foreground
 *            change current printing characters foreground color to fg
 */
void tgt_chfg(int fg)
{
    if(fg & TGT_COLOR_HIGHBRIGHT) tgt_chattr(TGT_TA_BOLD,0, 0); else tgt_chattr(TGT_TA_NORMAL,0, 0);
    fg=fg & TGT_COLOR_HIGHBRIGHTMASK;
    tgt_chattr(TGT_TA_FGCOLOR,fg, 0);
}
