/* ----------------------------------------------------------------------------
**      queue.c  simple message queues mechanism implementation
** ----------------------------------------------------------------------------
**      TGT                                       TextGUIToolkit Library
**      Copyright (C)   2000-2001       Mateusz Golicz and Tomasz Sterna
**
**      LGPL license
**
** ----------------------------------------------------------------------------
**      MAINTAINER      Mateusz Golicz
**
**      Log:
**      - cleanup
**
*/

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include "tgt.h"
#include "tgt_app.h"
#ifdef TGT_QUEUES

/*
 *
 * tgt_app_createqueue: Create a message queue
 *
 */

struct tgt_queue * tgt_app_createqueue(void)
{
    struct tgt_queue *ret;
    int pipes[2];
    
    ret=(struct tgt_queue*) malloc(sizeof(struct tgt_queue)); 
    pipe(pipes); /* get pipe fd's */
    ret->recv_pipe=pipes[0];
    ret->send_pipe=pipes[1];
    return(ret);
}

/*
 *
 * tgt_app_queue: Send a message to queue
 *
 */
void tgt_app_queue(struct tgt_queue *q, /* queue ptr from createqueue */
		   int code, /* user data */
		   long d1,  /* user data */ 
		   void *d2) /* user data */
{
    struct tgt_queue_msg m;
    m.code=code;
    m.data=d1;
    m.pointer=d2;
    write(q->send_pipe,&m,sizeof(struct tgt_queue_msg));
}

/*
 *
 * tgt_app_waitmsg: Wait for a message to arrive to a queue 
 *
 */
int tgt_app_waitmsg(struct tgt_queue *q,struct tgt_queue_msg *m)
{
    read(q->recv_pipe,m,sizeof(struct tgt_queue_msg));
    return(1);
}

/*
 *
 * tgt_app_checkmsg: Timeouted waitmsg
 *
 */
int tgt_app_checkmsg(struct tgt_queue *q,struct tgt_queue_msg *m,unsigned int s,unsigned int us)
{
    struct timeval tv;
    fd_set fds;
    int r;
    FD_ZERO(&fds); FD_SET(q->recv_pipe,&fds);
    tv.tv_sec=s; tv.tv_usec=us; /* wait no longer than s secs and us microsecs */
    r=select(q->recv_pipe+1,&fds,NULL,NULL,&tv); /* wait for action */
    if(r==0) return(0); /* timed out ! */
    read(q->recv_pipe,m,sizeof(struct tgt_queue_msg)); /* otherwise - read the message */
    return(1);
}

#endif
