/* ----------------------------------------------------------------------------
**      util.c  utility functions for lazy people
** ----------------------------------------------------------------------------
**      TGT                                       TextGUIToolkit Library
**      Copyright (C)   2000-2001       Mateusz Golicz and Tomasz Sterna
**
**      LGPL license
**
** ----------------------------------------------------------------------------
**      MAINTAINER      Mateusz Golicz
**
**      Log:
**      - cleanup
**
*/

#include "tgt.h"
#include <string.h>
#include <setjmp.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#define min(a,b) ((a) > (b) ? (b) : (a))
#endif

#ifndef TGT_NOPTHREAD
extern int async_running;
#endif

extern int async_semaphore;
jmp_buf jb;

int tgt_stopper(struct tgt_object *obj)
{
    return(1);
}

/* This function is called when user selects one of buttons in the dialog window */

int util_req_btc(struct tgt_object *obj)
{
    longjmp(jb,obj->id+1);
}


/*
 *
 * tgt_easyrequest: Display a message and allow user to make choice between some buttons
 *
 */
 
int tgt_easyrequest(struct tgt_object *ref, /* reference object */
		    char *title, /* dialog title */
		    char *body,  /* text */
		    char *buttons) /* |-separated list of texts to be placed inside buttons */
{
    int xs,ys,xc,yc;
    char **linepointers;
    char *str,*stra;
    int k,i,ml,b;
    struct tgt_object *req;
    struct tgt_object *bt;
    void *kn,*kp;
    int id;
    
    linepointers=(char **) malloc(sizeof(char*)*ref->ys);

    /* break the text into lines */
    for(k=0,ml=0,str=body=strdup(body);;k++)
    {
	char * newstr;
	int len;
	
	if(k>(ref->ys-5)) break; /* eerm, too long ! */
	linepointers[k]=str; /* current line is current line */
	
	if((newstr = index(str, '\n')))
	    len = newstr - str;
	else
	    len = strlen(str);
	
	if(len>(ref->xs-5))
	{
	    int k = ref->xs - 4;
	    for(; k>0 && str[k]!=' '; k--);
	    if(k == 0) tgt_fatal_error("tgt_easyrequest(): Unable to cut the break the string, terminal too small to hold such a big one");
	    str += k;
	}
	else
	{
	    if(!(str = newstr))  /* .. if there is no \n around ... */
	    { 
		ml=max(ml,strlen(linepointers[k])); /* correct maximal size variable */
		break;
	    }
	}
	*str=0; /* replace \n with \0 */
	ml=max(ml,strlen(linepointers[k])); str++; /* correct maximal size */
	if(*str==0) break;
    }
    k+=1;
    xs=min(max(max(strlen(title)+6,ml+4),strlen(buttons)+4),ref->xs); 
    /* calculate the X size of our window */
    if(xs>ref->xs) return(-1); /* too large ? */
    
    ys=k+5;
    if(ys>ref->ys) return(-1); /* same with Y-size */
    
    xc=(ref->xs-xs)/2; yc=(ref->ys-ys)/2; /* calculate coordinates, so our
	    window is beautyfully centered on the screen */

    /* open the window */
    req=tgt_createobject(TGT_CLASS_WINDOW,(tagitem[]) {TGTT_CALLBACK,(tgtt) &tgt_stopper,TGTT_END,0});

    tgt_set(req,TGTT_X,xc); tgt_set(req,TGTT_Y,yc);
    tgt_set(req,TGTT_XS,xs); tgt_set(req,TGTT_YS,ys);
    tgt_set(req,TGTT_WINDOW_TITLE,(tgtt) title);
    /* ... and set its parameters */
    
    buttons=strdup(buttons);
    for(b=1,str=buttons;(str=index(str+1,'|'));b++); /* calculate buttons # */
    xc=2+(xs-4-strlen(buttons))/2; /* calculate offset from left edge, so they're centered */
    for(str=buttons,i=0;i<b;i++)
    {
	if((stra=index(str,'|'))) *stra=0; /* find next | */
	/* and create a button object for current button */
	bt=tgt_createandlink(req,TGT_CLASS_BUTTON,(tagitem[]) {TGTT_CALLBACK,(tgtt) util_req_btc,TGTT_END,0});
	tgt_set(bt,TGTT_X,xc); xc+=strlen(str)+1; /* set its parameters: offset from left edge */
	tgt_set(bt,TGTT_Y,ys-3);
	tgt_set(bt,TGTT_BUTTON_CAPTION,(tgtt) str);
	tgt_set(bt,TGTT_ID,i);
	if(stra) str=stra+1; else break; /* go get the next button */
    }
    for(i=0;i<k;i++) /* now, create objects for our text lines */
    {
	bt=tgt_createandlink(req,TGT_CLASS_LABEL,(tagitem[]) {TGTT_X,2,TGTT_END,0});
	tgt_set(bt,TGTT_Y,2+i);
	tgt_set(bt,TGTT_XS,xs-4);
	tgt_set(bt,TGTT_LABEL_TEXT,(tgtt) linepointers[i]);
    }

    kn=ref->next_keys; ref->next_keys=NULL;
    kp=ref->prev_keys; ref->prev_keys=NULL; /* we do not want to give a possibility to switch objects */
    
    if((id=setjmp(jb))) /* set jump here */
    {
	/* if we get here, it means longjmp was called */
	tgt_unlink(req); 
	ref->next_keys=kn; ref->prev_keys=kp; /* restore switching */
	tgt_destroyobject(req); /* destroy window */
	free(body); free(linepointers); free(buttons); /* clean up */
	async_semaphore=1; /* allow async critical actions */
	tgt_refresh(ref);
	return(id-1);
    }
    
    tgt_link(req,ref); 
    tgt_setfocus(req); /* make our dialog visible */

#ifndef TGT_NOPTHREAD
    if(async_running) async_semaphore=0; /* forbid async critical actions */
#endif    
    while(1) tgt_getkey(req); /* wait for keys */
}

/*
 *
 * tgt_askstring: Ask user for string input
 *
 */

int tgt_askstring(struct tgt_object *ref,char *title,char *body,char *buttons,char *buffer,int maxlen)
{
    int xs,ys,xc,yc;
    char **linepointers;
    char *str,*stra;
    int k,i,ml,b;
    struct tgt_object *req;
    struct tgt_object *bt;
    struct tgt_object *strb;
    void *kn,*kp;
    int id;
    
    linepointers=(char **) malloc(sizeof(char*)*ref->ys);

    for(k=0,ml=0,str=body=strdup(body);;k++)
    {
	char * newstr;
	int len;
	
	if(k>(ref->ys-5)) break; /* eerm, too long ! */
	linepointers[k]=str; /* current line is current line */
	
	if((newstr = index(str, '\n')))
	    len = newstr - str;
	else
	    len = strlen(str);
	
	if(len>(ref->xs-5))
	{
	    int k = ref->xs - 4;
	    for(; k>0 && str[k]!=' '; k--);
	    if(k == 0) tgt_fatal_error("tgt_easyrequest(): Unable to cut the break the string, terminal too small to hold such a big one");
	    str += k;
	}
	else
	{
	    if(!(str = newstr))  /* .. if there is no \n around ... */
	    { 
		ml=max(ml,strlen(linepointers[k])); /* correct maximal size variable */
		break;
	    }
	}
	*str=0; /* replace \n with \0 */
	ml=max(ml,strlen(linepointers[k])); str++; /* correct maximal size */
	if(*str==0) break;
    }
    k+=1;
    xs=min(max(max(strlen(title)+6,ml+4),strlen(buttons)+4),ref->xs); if(xs>ref->xs) return(-1);
    ys=k+8; if(ys>ref->ys) return(-1);
    xc=(ref->xs-xs)/2; yc=(ref->ys-ys)/2;

    req=tgt_createobject(TGT_CLASS_WINDOW,(tagitem[]) {TGTT_CALLBACK,(tgtt) &tgt_stopper,TGTT_END,0});
    tgt_set(req,TGTT_X,xc); tgt_set(req,TGTT_Y,yc);
    tgt_set(req,TGTT_XS,xs); tgt_set(req,TGTT_YS,ys);
    tgt_set(req,TGTT_WINDOW_TITLE,(tgtt) title);

    strb=tgt_createandlink(req,TGT_CLASS_STRING,(tagitem[]) {TGTT_X,2,TGTT_END,0});
    tgt_set(strb,TGTT_Y,ys-5);
    tgt_set(strb,TGTT_XS,xs-4);
    tgt_set(strb,TGTT_STRING_MAX,maxlen);
    tgt_set(strb,TGTT_STRING_STRING,(tgtt) buffer);
    
    buttons=strdup(buttons);
    for(b=1,str=buttons;(str=index(str+1,'|'));b++);   
    xc=2+(xs-4-strlen(buttons))/2;
    for(str=buttons,i=0;i<b;i++)
    {
	if((stra=index(str,'|'))) *stra=0;
	bt=tgt_createandlink(req,TGT_CLASS_BUTTON,(tagitem[]) {TGTT_CALLBACK,(tgtt) util_req_btc,TGTT_END,0});
	tgt_set(bt,TGTT_X,xc); xc+=strlen(str)+1;
	tgt_set(bt,TGTT_Y,ys-3);
	tgt_set(bt,TGTT_BUTTON_CAPTION,(tgtt) str);
	tgt_set(bt,TGTT_ID,i);
	if(stra) str=stra+1; else break;
    }


    for(i=0;i<k;i++)
    {
	bt=tgt_createandlink(req,TGT_CLASS_LABEL,(tagitem[]) {TGTT_X,2,TGTT_END,0});
	tgt_set(bt,TGTT_Y,2+i);
	tgt_set(bt,TGTT_XS,xs-4);
	tgt_set(bt,TGTT_LABEL_TEXT,(tgtt) linepointers[i]);
    }

    kn=ref->next_keys; ref->next_keys=NULL;
    kp=ref->prev_keys; ref->prev_keys=NULL;
    
    if((id=setjmp(jb)))
    {
	tgt_get(strb,TGTT_STRING_STRING,(tgtt*) (&str));
	snprintf(buffer,maxlen,"%s",str);
	tgt_unlink(req);
	ref->next_keys=kn; ref->prev_keys=kp;
	tgt_destroyobject(req);
	free(body); free(linepointers); free(buttons);
	async_semaphore=1;
	tgt_refresh(ref);
	return(id-1);
    }
    
    tgt_link(req,ref);
    tgt_setfocus(req);

#ifndef TGT_NOPTHREAD
    if(async_running) async_semaphore=0;
#endif
    while(1) tgt_getkey(req);
}
