/* ----------------------------------------------------------------------------
**      tgmlp.c TextGui Markup Language parser
** ----------------------------------------------------------------------------
**      TGT                                       TextGUIToolkit Library
**      Copyright (C)   2000-2001       Mateusz Golicz and Tomasz Sterna
**
**      LGPL license
**
** ----------------------------------------------------------------------------
**      MAINTAINER      Mateusz Golicz
**
**      Log:
**
**
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "tgmlp.h"

struct tgml_parserdata td = { 0, 0, 1, NULL, 0};

void parse_tgml(unsigned char * file, int length)
{
    struct tgml_taginfo *tag;
    int pos;
    
    
    printf("/* Auto-generated by tgmlp , the Text GUI Markup Language parser, */\n");
    printf("/* a part of TGT package. (C) 2000-2001 Mateusz Golicz & Tomasz Sterna */\n");
    printf("\n#include \"tgt/tgt.h\"\n");
    printf("#include <stdio.h>\n\n");
    
    for(pos = 0;;)
    {
	/* 1 - skip to beginning of a tag */
	for(;file[pos] != '<' && pos<length; pos++);

	if(pos >= length) break;
//	printf("Found < @ %d\n",pos);
	/* 2 - fetch that tag */
	tag = tgml_fetchtag(file+pos, length-pos);
	if(! tag)
	{
	    fprintf(stderr, "Trouble processing file at byte %d\n",pos);
	    exit(1);
	}
//	printf("Got %stag %s len %d\n",tag->negative?"negative ":"",tag->tag_name, tag->length);
	parsetag(tag);
	pos += tag->length;
	tgml_destroyti(tag);
    }
}

int main(int argc, char ** argv)
{
    int fd, length, current, p;
    unsigned char * mem;
    
    if(argc < 2 || argc > 3)
    {
	fprintf(stderr, "Usage: %s <input-file> [output-file]\n", argv[0]);
	exit(0);
    }
    if(argc == 3)
    {
	fd = open(argv[2], O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if(fd == -1)
        {
	    perror("open");
	    fprintf(stderr, "Unable to open output file\n");
	    exit(0);
	}
	dup2(fd, 1);
    }
    /* load file into memory */
    fd = open(argv[1], O_RDONLY);
    if(fd == -1)
    {
	perror("open");
	fprintf(stderr, "Unable to open input file\n");
	exit(0);
    }
    if((length = lseek(fd, 0, SEEK_END)) == -1)
    {
	perror("lseek");
	close(fd);
	fprintf(stderr, "Unable measure input length\n");
	exit(0);
    }
    if(lseek(fd, 0, SEEK_SET) == -1)
    {
	perror("lseek");
	close(fd);
	fprintf(stderr, "Unable measure input length\n");
	exit(0);
    }
    mem = (unsigned char *) malloc(length);
    
    for(current = 0; current<length; )
    {
	p = read(fd, mem + current, length - current);
	current += p;
	if(p < 1)
	{
	    perror("read");
	    fprintf(stderr, "Trouble reading file into memory\n");
	    exit(0);
	} 
    }
    close(fd);
    
    /* and pass it to analysing functions */
    parse_tgml(mem, length);
    return(0);
}
